<?php

/**
 * Created by PhpStorm.
 * Script Name: Bgftask.php
 * Create: 2020/5/24 上午10:25
 * Description: 定时任务
 * Author: fudaoji<fdj@kuryun.cn>
 */
namespace app\admin\controller;

use app\common\model\bgf\AgentGoods;
use app\common\model\bgf\Task;

class Bgftask extends Botbase
{
    /**
     * @var Task
     */
    protected $model;
    /**
     * @var array
     */
    private $tabList;

    /**
     * @var AgentGoods
     */
    private $agentGoodsM;

    public function initialize()
    {
        parent::initialize(); // TODO: Change the autogenerated stub
        $this->model = new Task();
        $this->agentGoodsM = new AgentGoods();
        $this->tabList = [
            'todo' => [
                'title' => '待发送',
                'href' => url('index', ['name' => 'todo'])
            ],
            'done' => [
                'title' => '已发送',
                'href' => url('index', ['name' => 'done'])
            ],
        ];
    }

    public function index()
    {
        $name = input('name', 'todo');
        if (request()->isPost()) {
            $post_data = input('post.');
            $where = ['bot_id' => $this->bot['id']];
            !empty($post_data['search_key']) && $where['remark|goods_title'] = ['like', '%' . $post_data['search_key'] . '%'];
            //isset($post_data['complete']) && $post_data['complete'] >= 0 && $where['complete_time'] = $post_data['complete'] == 0 ? 0 : ['>', 0];
            if($name == 'todo'){
                $where['complete_time'] = 0;
                $order = ['plan_time' => 'asc'];
            }else{
                $where['complete_time'] = ['>', 0];
                $order = ['complete_time' => 'desc'];
            }

            $params = [
                'alias' => 'task',
                'join' => [
                    ['bgf_agent_goods goods', 'goods.goods_id=task.goods_id']
                ],
                'where' => $where,
                'refresh' => true
            ];
            $total = $this->model->totalJoin($params);
            if ($total) {
                $list = $this->model->getListJoin(array_merge($params, [
                    'limit' => [$post_data['page'], $post_data['limit']],
                    'order' => $order,
                    'field' => ['task.*', 'goods.goods_title']
                ]));
                foreach ($list as $k => $v){
                    $ids = explode(',', $v['wxids']);
                    if($member = model('admin/botMember')->getOneByMap([
                        'wxid' => $ids[0]
                    ])){
                        $v['members'] = $member['nickname'];
                        if(count($ids) > 1){
                            $v['members'] .= "等".count($ids)."个对象";
                        }
                    }else{
                        $v['members'] = "--";
                    }
                    $list[$k] = $v;
                }
            }else{
                $list = [];
            }

            $this->success('success', '', ['total' => $total, 'list' => $list]);
        }

        $builder = new ListBuilder();
        $builder->setSearch([
            //['type' => 'select', 'name' => 'complete', 'title' => '发送状态', 'options' => [-1 => '全部', 0 => '未发送', 1 => '已发送']],
            ['type' => 'text', 'name' => 'search_key', 'title' => '关键词', 'placeholder' => '内容']
        ])
            ->setTabNav($this->tabList, $name)
            ->setDataUrl(url('index', ['name' => $name]))
            //->addTopButton('addnew', ['title' => '快速添加', 'href' => url('quickAdd')])
            ->addTopButton('addnew', ['title' => '添加任务'])
            ->addTableColumn(['title' => '发送顺序', 'field' => 'id', 'type' => 'index'])
            ->addTableColumn(['title' => '计划发送时间', 'field' => 'plan_time', 'minWidth' => 180, 'type' => 'datetime'])
            ->addTableColumn(['title' => '发送商品', 'field' => 'goods_title', 'minWidth' => 100])
            ->addTableColumn(['title' => '接收对象', 'field' => 'members', 'minWidth' => 200])
            ->addTableColumn(['title' => '备注', 'field' => 'remark', 'minWidth' => 60]);
        if($name == 'done'){
            $builder->addTableColumn(['title' => '完成时间', 'field' => 'complete_time', 'minWidth' => 180, 'type' => 'datetime']);
        }else{
            $builder->addTableColumn(['title' => '是否开启', 'field' => 'status', 'minWidth' => 70, 'type' => 'switch', 'options' => [0 => '停止', 1 => '开启']])
                ->addTableColumn(['title' => '操作', 'minWidth' => 120, 'type' => 'toolbar'])
                ->addRightButton('edit')
                ->addRightButton('delete');
        }

        return $builder->show();
    }

    /**
     * 新增任务
     * @return mixed
     * @throws \think\db\exception\DbException
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function add(){
        $data = [

        ];
        $last_one = $this->model->getOneByOrder([
            'where' => ['admin_id' => $this->adminInfo['id'], 'bot_id' => $this->bot['id']],
            'field' => 'wxids',
            'order' => ['id' => 'desc']
        ]);
        if($last_one){
            $data['wxids'] = explode(',', $last_one['wxids']);
        }

        $groups = model('admin/botMember')->getField('wxid,nickname',['uin' => $this->bot['uin']]);
        $builder = new FormBuilder();
        $builder->setPostUrl(url('savePost'))
            ->addFormItem('goods_id', 'chosen', '选择商品', '选择商品', $this->getGoodsList(), 'required')
            ->addFormItem('wxids', 'chosen_multi', '接收对象', '发送对象', $groups, 'required')
            ->addFormItem('plan_time', 'datetime', '发送时间', '不填则根据设置的发单间隔时间确定', [], '')
            ->addFormItem('remark', 'text', '备注', '')
            ->setFormData($data);
        return $builder->show();
    }

    private function getGoodsList(){
        return $this->agentGoodsM->getIdToTitle();
    }

    /**
     * 编辑任务
     * @return mixed
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\db\exception\DbException
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function edit(){
        $id = input('id', 0, 'intval');
        if(!$data = $this->model->getOne($id)){
            $this->error('数据不存在');
        }
        $data['wxids'] = explode(',', $data['wxids']);
        $groups = model('admin/botMember')->getField('wxid,nickname',['uin' => $this->bot['uin']]);
        $builder = new FormBuilder();
        $builder->setPostUrl(url('savePost'))
            ->addFormItem('id', 'hidden', 'id', 'id')
            ->addFormItem('goods_id', 'chosen', '选择商品', '选择商品', $this->getGoodsList(), 'required')
            ->addFormItem('wxids', 'chosen_multi', '接收对象', '发送对象', $groups, 'required')
            ->addFormItem('plan_time', 'datetime', '发送时间', '不填则根据设置的发单间隔时间确定', [], '')
            ->addFormItem('remark', 'text', '备注', '')
            ->setFormData($data);
        return $builder->show();
    }

    public function savePost($jump = '', $data = []){
        $data = input('post.');
        $plan_time = empty($data['plan_time']) ? time() : strtotime($data['plan_time']);
        $data['plan_time'] = $plan_time;
        if(empty($data['goods_id'])){
            $this->error('请选择要发送的商品!');
        }
        $data['bot_id'] = $this->bot['id'];
        return parent::savePost($jump, $data);
    }
}